@@ -18,32 +18,40 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+#ifndef lint
+static const char rcsid[] =
+    "@(#) $Header: /tcpdump/master/tcpslice/gwtm2secs.c,v 1.7 2001/10/30 17:59:46 fenner Exp $ (LBL)";
+#endif
 
 /*
  * gwtm2secs.c - convert "tm" structs for Greenwich time to Unix timestamp
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <sys/types.h>
 
 #include <time.h>
+#include <pcap.h>
 
+#include "gnuc.h"
 #ifdef HAVE_OS_PROTO_H
 #include "os-proto.h"
 #endif
 
 #include "tcpslice.h"
 
-const int days_in_month[] =
+static int days_in_month[] =
 	/* Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
 	{  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-time_t
-gwtm2secs( const struct tm *tm )
-{
+#define IS_LEAP_YEAR(year)	\
+	(year % 4 == 0 && (year % 100 != 0 || year % 400 == 0))
+
+time_t gwtm2secs( struct tm *tm )
+	{
 	int i, days, year;
 
 	/* tm_year is either:
@@ -62,11 +70,11 @@ gwtm2secs( const struct tm *tm )
 
 	days = 0;
 	for ( i = 1970; i < year; ++i )
-	{
+		{
 		days += 365;
 		if ( IS_LEAP_YEAR(i) )
 			++days;
-	}
+		}
 
 	for ( i = 0; i < tm->tm_mon; ++i )
 		days += days_in_month[i];
@@ -77,4 +85,4 @@ gwtm2secs( const struct tm *tm )
 	days += tm->tm_mday - 1; /* -1 since days are numbered starting at 1 */
 
 	return days * 86400 + tm->tm_hour * 3600 + tm->tm_min * 60 + tm->tm_sec;
-}
+	}
