@@ -19,38 +19,31 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
+#ifndef lint
+static const char rcsid[] =
+    "@(#) $Header: /tcpdump/master/tcpslice/machdep.c,v 1.5 2001/10/30 17:59:46 fenner Exp $ (LBL)";
 #endif
 
-#include <stddef.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
+#include <sys/types.h>
 #ifdef __osf__
-#include <stdio.h>
 #include <sys/sysinfo.h>
 #include <sys/proc.h>
-#endif /* __osf__ */
+#endif
 
-#include "varattrs.h"
 #include "machdep.h"
 
-/*
- * On platforms where the CPU doesn't support unaligned loads, force
- * unaligned accesses to abort with SIGBUS, rather than being fixed
- * up (slowly) by the OS kernel; on those platforms, misaligned accesses
- * are bugs, and we want tcpslice to crash so that the bugs are reported.
- *
- * The only OS on which this is necessary is DEC OSF/1^W^WDigital
- * UNIX^W^WTru64 UNIX.
- */
 int
-abort_on_misalignment(char *ebuf _U_, const size_t ebufsiz _U_)
+abort_on_misalignment(char *ebuf)
 {
 #ifdef __osf__
 	static int buf[2] = { SSIN_UACPROC, UAC_SIGBUS };
 
 	if (setsysinfo(SSI_NVPAIRS, (caddr_t)buf, 1, 0, 0) < 0) {
-		(void)snprintf(ebuf, ebufsiz, "setsysinfo: errno %d", errno);
+		(void)sprintf(ebuf, "setsysinfo: errno %d", errno);
 		return (-1);
 	}
 #endif
