@@ -18,65 +18,58 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+#ifndef lint
+static const char rcsid[] =
+    "@(#) $Header: /tcpdump/master/tcpslice/search.c,v 1.20 2001/11/15 01:17:40 fenner Exp $ (LBL)";
+#endif
 
 /*
- * search.c - supports fast searching through pcap files for timestamps
+ * search.c - supports fast searching through tcpdump files for timestamps
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <sys/types.h>
 
 #include <pcap.h>
+#ifdef HAVE_PCAP_INT_H
+#include <pcap-int.h>		/* because we are directly reading the file */
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 
+#include "gnuc.h"
 #ifdef HAVE_OS_PROTO_H
 #include "os-proto.h"
 #endif
 
 #include "tcpslice.h"
 
-/*
- * We directly read a pcap file, so declare the per-packet header
- * ourselves.  It's not platform-dependent or version-dependent;
- * if the per-packet header doesn't look *exactly* like this, it's
- * not a valid pcap file.
+#ifndef HAVE_PCAP_INT_H
+/* If we're building without pcap's internal headers, we have to just
+ * assume that the public pcap_pkthdr is the same as is written to
+ * the file.  This can cause problems on systems with 64-bit longs;
+ * the symptom is that tcpslice can't find any packets to read.
+ * The solution is to build with the pcap-int.h from the installed
+ * version of libpcap.
  */
+#define	pcap_sf_pkthdr	pcap_pkthdr
+#endif
 
-/*
- * This is a timeval as stored in a savefile.
- * It has to use the same types everywhere, independent of the actual
- * `struct timeval'; `struct timeval' has 32-bit tv_sec values on some
- * platforms and 64-bit tv_sec values on other platforms, and writing
- * out native `struct timeval' values would mean files could only be
- * read on systems with the same tv_sec size as the system on which
- * the file was written.
- */
-struct pcap_timeval {
-    bpf_int32 tv_sec;		/* seconds */
-    bpf_int32 tv_usec;		/* microseconds */
-};
-
-/*
- * This is a `pcap_pkthdr' as actually stored in a savefile.
- */
-struct pcap_sf_pkthdr {
-    struct pcap_timeval ts;	/* time stamp */
-    bpf_u_int32 caplen;		/* length of portion present */
-    bpf_u_int32 len;		/* length this packet (off wire) */
-};
+/* stringify macros, for error reporting. */
+#define	SS(x)	#x
+#define	S(x)	SS(x)
 
 /* Maximum number of seconds that we can conceive of a dump file spanning. */
 #define MAX_REASONABLE_FILE_SPAN (3600*24*366)	/* one year */
 
 /* Maximum packet length we ever expect to see. */
-#define MAX_REASONABLE_PACKET_LENGTH 262144
+#define MAX_REASONABLE_PACKET_LENGTH 65535
 
 /* Size of a packet header in bytes; easier than typing the sizeof() all
  * the time ...
@@ -115,8 +108,8 @@ extern int snaplen;
  * if the header looks reasonable and zero otherwise.
  */
 static int
-reasonable_header( const struct pcap_pkthdr *hdr, const time_t first_time, time_t last_time )
-{
+reasonable_header( struct pcap_pkthdr *hdr, time_t first_time, time_t last_time )
+	{
 	if ( last_time == 0 )
 		last_time = first_time + MAX_REASONABLE_FILE_SPAN;
 
@@ -126,15 +119,19 @@ reasonable_header( const struct pcap_pkthdr *hdr, const time_t first_time, time_
 	       hdr->len <= MAX_REASONABLE_PACKET_LENGTH &&
 	       hdr->caplen > 0 &&
 	       hdr->caplen <= MAX_REASONABLE_PACKET_LENGTH;
-}
+	}
+
 
 #define	SWAPLONG(y) \
 ((((y)&0xff)<<24) | (((y)&0xff00)<<8) | (((y)&0xff0000)>>8) | (((y)>>24)&0xff))
+#define	SWAPSHORT(y) \
+	( (((y)&0xff)<<8) | (((y)&0xff00)>>8) )
 
 /* Given a buffer, extracts a (properly aligned) packet header from it. */
+
 static void
-extract_header( pcap_t *p, const u_char *buf, struct pcap_pkthdr *hdr )
-{
+extract_header( pcap_t *p, u_char *buf, struct pcap_pkthdr *hdr )
+	{
 	struct pcap_sf_pkthdr sfhdr;
 
 	memcpy(&sfhdr, buf, sizeof(sfhdr));
@@ -145,12 +142,12 @@ extract_header( pcap_t *p, const u_char *buf, struct pcap_pkthdr *hdr )
 	hdr->len = sfhdr.len;
 
 	if ( pcap_is_swapped( p ) )
-	{
+		{
 		hdr->ts.tv_sec = SWAPLONG(hdr->ts.tv_sec);
 		hdr->ts.tv_usec = SWAPLONG(hdr->ts.tv_usec);
 		hdr->len = SWAPLONG(hdr->len);
 		hdr->caplen = SWAPLONG(hdr->caplen);
-	}
+		}
 
 	/*
 	 * From bpf/libpcap/savefile.c:
@@ -167,7 +164,8 @@ extract_header( pcap_t *p, const u_char *buf, struct pcap_pkthdr *hdr )
 		hdr->caplen = hdr->len;
 		hdr->len = t;
 		}
-}
+	}
+
 
 /* Search a buffer to locate the first header within it.  Return values
  * are HEADER_NONE, HEADER_CLASH, HEADER_PERHAPS, and HEADER_DEFINITELY.
@@ -202,10 +200,10 @@ extract_header( pcap_t *p, const u_char *buf, struct pcap_pkthdr *hdr )
 #define HEADER_DEFINITELY 3
 
 static int
-find_header( pcap_t *p, u_char *buf, const int buf_len,
-		const time_t first_time, const time_t last_time,
+find_header( pcap_t *p, u_char *buf, int buf_len,
+		time_t first_time, time_t last_time,
 		u_char **hdrpos_addr, struct pcap_pkthdr *return_hdr )
-{
+	{
 	u_char *bufptr, *bufend, *last_pos_to_try;
 	struct pcap_pkthdr hdr, hdr2;
 	int status = HEADER_NONE;
@@ -220,22 +218,22 @@ find_header( pcap_t *p, u_char *buf, const int buf_len,
 	last_pos_to_try = bufend - PACKET_HDR_LEN;
 
 	for ( bufptr = buf; bufptr < last_pos_to_try; ++bufptr )
-	{
+	    {
 	    extract_header( p, bufptr, &hdr );
 
 	    if ( reasonable_header( &hdr, first_time, last_time ) )
-	    {
+		{
 		u_char *next_header = bufptr + PACKET_HDR_LEN + hdr.caplen;
 
 		if ( next_header + PACKET_HDR_LEN < bufend )
-		{ /* check for another good header */
+		    { /* check for another good header */
 		    extract_header( p, next_header, &hdr2 );
 
 		    if ( reasonable_header( &hdr2, hdr.ts.tv_sec,
 			    hdr.ts.tv_sec + MAX_REASONABLE_HDR_SEPARATION ) )
-		    { /* a confirmed header */
+			{ /* a confirmed header */
 			switch ( status )
-			{
+			    {
 			    case HEADER_NONE:
 			    case HEADER_PERHAPS:
 				status = HEADER_DEFINITELY;
@@ -253,18 +251,19 @@ find_header( pcap_t *p, u_char *buf, const int buf_len,
 				return HEADER_CLASH;
 
 			    default:
-				error( "bad status in %s()", __func__ );
+				error( "bad status in find_header()" );
+			    }
 			}
-		    }
 
 		    /* ... else the header is bogus - we've verified that it's
 		     * not followed by a reasonable header.
 		     */
-		}
+		    }
+
 		else
-		{ /* can't check for another good header */
+		    { /* can't check for another good header */
 		    switch ( status )
-		    {
+			{
 			case HEADER_NONE:
 			    status = HEADER_PERHAPS;
 			    *hdrpos_addr = bufptr;
@@ -284,17 +283,18 @@ find_header( pcap_t *p, u_char *buf, const int buf_len,
 			    break;
 
 			default:
-			    error( "bad status in %s()", __func__ );
+			    error( "bad status in find_header()" );
+			}
 		    }
 		}
 	    }
-	}
 
 	if ( status == HEADER_PERHAPS && saw_PERHAPS_clash )
 		status = HEADER_CLASH;
 
 	return status;
-}
+	}
+
 
 /* Positions the sf_readfile stream such that the next sf_read() will
  * read the final full packet in the file.  Returns non-zero if
@@ -307,11 +307,11 @@ find_header( pcap_t *p, u_char *buf, const int buf_len,
  * present in the dump file.
  */
 int
-sf_find_end( pcap_t *p, const struct timeval *first_timestamp,
+sf_find_end( pcap_t *p, struct timeval *first_timestamp,
 		struct timeval *last_timestamp )
-{
+	{
 	time_t first_time = first_timestamp->tv_sec;
-	int64_t len_file;
+	off_t len_file;
 	int num_bytes;
 	u_char *buf, *bufpos, *bufend;
 	u_char *hdrpos;
@@ -319,14 +319,11 @@ sf_find_end( pcap_t *p, const struct timeval *first_timestamp,
 	int status;
 
 	/* Find the length of the file. */
-	if ( fseek64( pcap_file (p), (int64_t) 0, SEEK_END ) < 0 )
+	if ( FSEEK( pcap_file (p), (off_t) 0, SEEK_END ) < 0 )
 		return 0;
 
-	len_file = ftell64( pcap_file (p) );
-	if ( len_file < 0 )
-		return 0;
-	/* Casting a non-negative int64_t to uint64_t always works as expected. */
-	if ( (uint64_t) len_file < MAX_BYTES_FOR_DEFINITE_HEADER )
+	len_file = FTELL( pcap_file (p) );
+	if ( len_file < MAX_BYTES_FOR_DEFINITE_HEADER )
 		num_bytes = len_file;
 	else
 		num_bytes = MAX_BYTES_FOR_DEFINITE_HEADER;
@@ -336,7 +333,7 @@ sf_find_end( pcap_t *p, const struct timeval *first_timestamp,
 	 * finding a "definite" header and following its chain to the
 	 * end of the file.
 	 */
-	if ( fseek64( pcap_file( p ), -(int64_t) num_bytes, SEEK_END ) < 0 )
+	if ( FSEEK( pcap_file( p ), -(off_t) num_bytes, SEEK_END ) < 0 )
 		return 0;
 
 	buf = (u_char *)malloc((u_int) num_bytes);
@@ -358,7 +355,7 @@ sf_find_end( pcap_t *p, const struct timeval *first_timestamp,
 	 * chain till we find the last valid packet in the file ...
 	 */
 	for ( ; ; )
-	{
+		{
 		/* move to the next header position */
 		bufpos = hdrpos + PACKET_HDR_LEN + hdr.caplen;
 
@@ -394,81 +391,91 @@ sf_find_end( pcap_t *p, const struct timeval *first_timestamp,
 		/* Accept this packet as fully legit. */
 		hdrpos = bufpos;
 		hdr = successor_hdr;
-	}
+		}
 
 	/* Success!  Last valid packet is at hdrpos. */
-	TIMEVAL_FROM_PKTHDR_TS(*last_timestamp, hdr.ts);
+	*last_timestamp = hdr.ts;
 	status = 1;
 
 	/* Seek so that the next read will start at last valid packet. */
-	if ( fseek64( pcap_file( p ), -(int64_t) (bufend - hdrpos), SEEK_END ) < 0 )
-		error( "final fseek64() failed in %s()", __func__ );
+	if ( FSEEK( pcap_file( p ), -(off_t) (bufend - hdrpos), SEEK_END ) < 0 )
+		error( "final " S(FSEEK) "() failed in sf_find_end()" );
 
     done:
 	free( (char *) buf );
 
 	return status;
-}
+	}
+
 
 /* Takes two timeval's and returns the difference, tv2 - tv1, as a double. */
+
 static double
-timeval_diff( const struct timeval *tv1, const struct timeval *tv2 )
-{
+timeval_diff( struct timeval *tv1, struct timeval *tv2 )
+	{
 	double result = (tv2->tv_sec - tv1->tv_sec);
 	result += (tv2->tv_usec - tv1->tv_usec) / 1000000.0;
 
 	return result;
-}
+	}
+
 
 /* Returns true if timestamp t1 is chronologically less than timestamp t2. */
+
 int
-sf_timestamp_less_than( const struct timeval *t1, const struct timeval *t2 )
-{
+sf_timestamp_less_than( struct timeval *t1, struct timeval *t2 )
+	{
 	return t1->tv_sec < t2->tv_sec ||
 	       (t1->tv_sec == t2->tv_sec &&
 	        t1->tv_usec < t2->tv_usec);
-}
+	}
+
 
 /* Given two timestamps on either side of desired_time and their positions,
  * returns the interpolated position of the desired_time packet.  Returns a
  * negative value if the desired_time is outside the given range.
  */
-static int64_t
-interpolated_position( const struct timeval *min_time, const int64_t min_pos,
-			const struct timeval *max_time, const int64_t max_pos,
-			const struct timeval *desired_time )
-{
+
+static off_t
+interpolated_position( struct timeval *min_time, off_t min_pos,
+			struct timeval *max_time, off_t max_pos,
+			struct timeval *desired_time )
+	{
 	double full_span = timeval_diff( max_time, min_time );
 	double desired_span = timeval_diff( desired_time, min_time );
-	int64_t full_span_pos = max_pos - min_pos;
+	off_t full_span_pos = max_pos - min_pos;
 	double fractional_offset = desired_span / full_span;
 
 	if ( fractional_offset < 0.0 || fractional_offset > 1.0 )
 		return -1;
 
-	return min_pos + (int64_t) (fractional_offset * (double) full_span_pos);
-}
+	return min_pos + (off_t) (fractional_offset * (double) full_span_pos);
+	}
+
 
 /* Reads packets linearly until one with a time >= the given desired time
  * is found; positions the dump file so that the next read will start
  * at the given packet.  Returns non-zero on success, 0 if an EOF was
  * first encountered.
  */
+
 static int
-read_up_to( pcap_t *p, const struct timeval *desired_time )
-{
+read_up_to( pcap_t *p, struct timeval *desired_time )
+	{
 	struct pcap_pkthdr hdr;
-	int64_t pos;
+	const u_char *buf;
+	off_t pos;
 	int status;
 
 	for ( ; ; )
-	{
-		struct timeval tvbuf;
-
-		pos = ftell64( pcap_file( p ) );
-
-		if ( pcap_next( p, &hdr ) == NULL )
 		{
+		struct timeval *timestamp;
+
+		pos = FTELL( pcap_file( p ) );
+		buf = pcap_next( p, &hdr );
+
+		if ( buf == 0 )
+			{
 			if ( feof( pcap_file( p ) ) )
 				{
 				status = 0;
@@ -476,23 +483,23 @@ read_up_to( pcap_t *p, const struct timeval *desired_time )
 				break;
 				}
 
-			error( "bad status in %s()", __func__ );
-		}
+			error( "bad status in read_up_to()" );
+			}
 
-		TIMEVAL_FROM_PKTHDR_TS(tvbuf, hdr.ts);
+		timestamp = &hdr.ts;
 
-		if ( ! sf_timestamp_less_than( &tvbuf, desired_time ) )
-		{
+		if ( ! sf_timestamp_less_than( timestamp, desired_time ) )
+			{
 			status = 1;
 			break;
+			}
 		}
-	}
 
-	if ( fseek64( pcap_file( p ), pos, SEEK_SET ) < 0 )
-		error( "fseek64() failed in %s()", __func__ );
+	if ( FSEEK( pcap_file( p ), pos, SEEK_SET ) < 0 )
+		error( S(FSEEK) "() failed in read_up_to()" );
 
 	return (status);
-}
+	}
 
 /* Positions the sf_readfile stream so that the next sf_read() will
  * return the first packet with a time greater than or equal to
@@ -507,21 +514,24 @@ read_up_to( pcap_t *p, const struct timeval *desired_time )
  * already aligned so that the next call to sf_next_packet() will yield
  * a valid packet.
  */
+
 int
 sf_find_packet( pcap_t *p,
-		struct timeval *min_time, int64_t min_pos,
-		struct timeval *max_time, int64_t max_pos,
-		const struct timeval *desired_time )
-{
+		struct timeval *min_time, off_t min_pos,
+		struct timeval *max_time, off_t max_pos,
+		struct timeval *desired_time )
+	{
 	int status = 1;
 	struct timeval min_time_copy, max_time_copy;
 	u_int num_bytes = MAX_BYTES_FOR_DEFINITE_HEADER;
+	int num_bytes_read;
+	off_t desired_pos, present_pos;
 	u_char *buf, *hdrpos;
 	struct pcap_pkthdr hdr;
 
 	buf = (u_char *) malloc( num_bytes );
 	if ( ! buf )
-		error( "malloc() failed in %s()", __func__ );
+		error( "malloc() failured in sf_find_packet()" );
 
 	min_time_copy = *min_time;
 	min_time = &min_time_copy;
@@ -530,12 +540,11 @@ sf_find_packet( pcap_t *p,
 	max_time = &max_time_copy;
 
 	for ( ; ; )	/* loop until positioned correctly */
-	{
-		int64_t desired_pos =
+		{
+		desired_pos =
 			interpolated_position( min_time, min_pos,
 					       max_time, max_pos,
 					       desired_time );
-		struct timeval tvbuf;
 
 		if ( desired_pos < 0 )
 			{
@@ -543,16 +552,14 @@ sf_find_packet( pcap_t *p,
 			break;
 			}
 
-		int64_t present_pos = ftell64( pcap_file( p ) );
-		if ( present_pos < 0 )
-			error ( "ftell64() failed in %s()", __func__ );
+		present_pos = FTELL( pcap_file( p ) );
 
 		if ( present_pos <= desired_pos &&
-		     (uint64_t) (desired_pos - present_pos) < STRAIGHT_SCAN_THRESHOLD )
-		{ /* we're close enough to just blindly read ahead */
+		     desired_pos - present_pos < STRAIGHT_SCAN_THRESHOLD )
+			{ /* we're close enough to just blindly read ahead */
 			status = read_up_to( p, desired_time );
 			break;
-		}
+			}
 
 		/* Undershoot the target a little bit - it's much easier to
 		 * then scan straight forward than to try to read backwards ...
@@ -561,10 +568,10 @@ sf_find_packet( pcap_t *p,
 		if ( desired_pos < min_pos )
 			desired_pos = min_pos;
 
-		if ( fseek64( pcap_file( p ), desired_pos, SEEK_SET ) < 0 )
-			error( "fseek64() failed in %s()", __func__ );
+		if ( FSEEK( pcap_file( p ), desired_pos, SEEK_SET ) < 0 )
+			error( S(FSEEK) "() failed in sf_find_packet()" );
 
-		int num_bytes_read =
+		num_bytes_read =
 			fread( (char *) buf, 1, num_bytes, pcap_file( p ) );
 
 		if ( num_bytes_read == 0 )
@@ -572,7 +579,7 @@ sf_find_packet( pcap_t *p,
 			 * undershoot, unless the dump file has only a
 			 * couple packets in it ...
 			 */
-			error( "fread() failed in %s()", __func__ );
+			error( "fread() failed in sf_find_packet()" );
 
 		if ( find_header( p, buf, num_bytes, min_time->tv_sec,
 				  max_time->tv_sec, &hdrpos, &hdr ) !=
@@ -584,28 +591,27 @@ sf_find_packet( pcap_t *p,
 		desired_pos += (hdrpos - buf);
 
 		/* Seek to the beginning of the header. */
-		if ( fseek64( pcap_file( p ), desired_pos, SEEK_SET ) < 0 )
-			error( "fseek64() failed in %s()", __func__ );
+		if ( FSEEK( pcap_file( p ), desired_pos, SEEK_SET ) < 0 )
+			error( S(FSEEK) "() failed in sf_find_packet()" );
 
-		TIMEVAL_FROM_PKTHDR_TS(tvbuf, hdr.ts);
-		if ( sf_timestamp_less_than( &tvbuf, desired_time ) )
-		{ /* too early in the file */
-			*min_time = tvbuf;
+		if ( sf_timestamp_less_than( &hdr.ts, desired_time ) )
+			{ /* too early in the file */
+			*min_time = hdr.ts;
 			min_pos = desired_pos;
-		}
+			}
 
-		else if ( sf_timestamp_less_than( desired_time, &tvbuf ) )
-		{ /* too late in the file */
-			*max_time = tvbuf;
+		else if ( sf_timestamp_less_than( desired_time, &hdr.ts ) )
+			{ /* too late in the file */
+			*max_time = hdr.ts;
 			max_pos = desired_pos;
-		}
+			}
 
 		else
 			/* got it! */
 			break;
-	}
+		}
 
 	free( (char *) buf );
 
 	return status;
-}
+	}
