@@ -19,57 +19,46 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+#ifndef lint
+static const char rcsid[] =
+    "@(#) $Header: /tcpdump/master/tcpslice/util.c,v 1.6 2001/10/30 17:59:46 fenner Exp $ (LBL)";
+#endif
+
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <stdarg.h>
+#include <pcap.h>
 #include <string.h>
 
 #ifdef TIME_WITH_SYS_TIME
 #include <time.h>
 #endif
 
+#include "gnuc.h"
 #ifdef HAVE_OS_PROTO_H
 #include "os-proto.h"
 #endif
 
 #include "tcpslice.h"
 
-static void
-complain(const char *fmt, va_list ap)
-{
-	(void)fprintf(stderr, "tcpslice: ");
-	(void)vfprintf(stderr, fmt, ap);
-	if (*fmt) {
-		fmt += strlen(fmt);
-		if (fmt[-1] != '\n')
-			(void)fputc('\n', stderr);
-	}
-}
-
-/* VARARGS */
-void
-warning(const char *fmt, ...)
-{
-	va_list ap;
-
-	va_start(ap, fmt);
-	complain(fmt, ap);
-	va_end(ap);
-}
-
 /* VARARGS */
 void
 error(const char *fmt, ...)
 {
 	va_list ap;
 
+	(void)fprintf(stderr, "tcpslice: ");
 	va_start(ap, fmt);
-	complain(fmt, ap);
+	(void)vfprintf(stderr, fmt, ap);
 	va_end(ap);
+	if (*fmt) {
+		fmt += strlen(fmt);
+		if (fmt[-1] != '\n')
+			(void)fputc('\n', stderr);
+	}
 	exit(1);
 	/* NOTREACHED */
 }
